@@ -30,6 +30,7 @@ from typing import Union
 import inspect
 import re
 import validators
+import importlib.util
 
 
 class PythonValidator:
@@ -48,7 +49,7 @@ class PythonValidator:
     """
 
     @staticmethod
-    def is_instance(
+    def is_instance_of(
         element: object,
         cls: Union[str, type, list]
     ) -> bool:
@@ -66,53 +67,37 @@ class PythonValidator:
         # paying attention to avoid infinite loops by
         # reusing other methods of this class
 
-        # 1. Single value and it is a class
-        if (
-            not PythonValidator.is_list(cls) and
-            isinstance(cls, type)
-        ):
-            return isinstance(element, cls)
+        # Force list to simplify checkings
+        cls = (
+            [cls]
+            if not PythonValidator.is_list(cls) else
+            cls
+        )
 
-        # 2. Single value and it is a string
-        if (
-            not PythonValidator.is_list(cls) and
-            isinstance(cls, str)
-        ):
-            return getattr(type(element), '__name__', None) == cls
-            
-        # 3. Multiple values and they are all classes
-        if (
-            PythonValidator.is_list(cls) and
-            all(
-                isinstance(cls_item, type)
-                for cls_item in cls
-            )
+        # 1. Multiple values and they are all classes
+        if all(
+            isinstance(cls_item, type)
+            for cls_item in cls
         ):
             return any(
                 isinstance(element, cls_item)
                 for cls_item in cls
             )
 
-        # 4. Multiple values and they are all strings
-        if (
-            PythonValidator.is_list(cls) and
-            all(
-                isinstance(cls_item, str)
-                for cls_item in cls
-            )
+        # 2. Multiple values and they are all strings
+        if all(
+            isinstance(cls_item, str)
+            for cls_item in cls
         ):
             return getattr(type(element), '__name__', None) in cls
 
-        # 5. Multiple values and they are mixed classes and strings
-        if (
-            PythonValidator.is_list(cls) and
-            all(
-                (
-                    isinstance(cls_item, str) or
-                    isinstance(cls_item, type)
-                )
-                for cls_item in cls
+        # 3. Multiple values and they are mixed classes and strings
+        if all(
+            (
+                isinstance(cls_item, str) or
+                isinstance(cls_item, type)
             )
+            for cls_item in cls
         ):
             element_class_name = getattr(type(element), '__name__', None)
 
@@ -127,14 +112,12 @@ class PythonValidator:
                 if isinstance(cls_item, type)
             ]
 
-            return any(
-                (
+            return (
+                any(
                     isinstance(element, cls_item)
                     for cls_item in cls_classes_list
                 ) or
-                (
-                    element_class_name in cls_str_list
-                )
+                element_class_name in cls_str_list
             )
         
         # TODO: Maybe some parameter provided is invalid
@@ -147,7 +130,7 @@ class PythonValidator:
         """
         Check if the provided 'element' is an instance of any class.
         """
-        return PythonValidator.is_instance(element, object)
+        return PythonValidator.is_instance_of(element, object)
         # This below is just another alternative I want to keep
         return getattr(element, '__name__', None) is None
     
@@ -211,14 +194,14 @@ class PythonValidator:
         """
         Check if the provided 'element' is a class.
         """
-        return PythonValidator.is_instance(element, type)
+        return PythonValidator.is_instance_of(element, type)
         # This below is just another alternative I want to keep
         return getattr(element, '__name__', None) is not None
     
     @staticmethod
-    def is_subclass(
+    def is_subclass_of(
         element: type,
-        cls: Union[str, type]
+        cls: Union[str, type, list]
     ) -> bool:
         """
         Check if the provided 'element' is a subclass of the provided
@@ -233,39 +216,84 @@ class PythonValidator:
         Consider this above to check later if your 'element' is an
         instance or a class.
         """
-        # TODO: Convert this to accept cls as array and check 
-        # diferent classes
         if (
-            not PythonValidator.is_string(cls) and
-            not PythonValidator.is_a_class(cls)
+            not PythonValidator.is_a_class(element) and
+            not PythonValidator.is_an_instance(element)
         ):
-            raise Exception(f'The provided "cls" parameter "{str(cls)}" is not a string nor a class.')
+            return False
         
-        if not PythonValidator.is_a_class(element):
-            # We want to know if it is a subclass, we don't
-            # care if the provided one is an instance or a class,
-            # so we try with both.
-            if PythonValidator.is_an_instance(element):
-                element = type(element)
-            else:
-                return False
-            
-        """
-        The element.__mro__ returns a list like this below in
-        which you can see, oredered, the classes hierarchy.
-        This can be useful, so I keep it here:
-
-        (<class 'yta_multimedia.video.generation.manim.classes.text.simple_text_manim_animation.ExampleManimAnimationGenerator'>, <class 'yta_multimedia.video.generation.manim.classes.base_manim_animation.BaseManimAnimation'>, <class 'manim.scene.scene.Scene'>, <class 'object'>)    
-        """
-            
-        return (
-            cls in [
-                base_class.__name__
-                for base_class in element.__bases__
-            ]
-            if PythonValidator.is_string(cls) else
-            issubclass(element, cls)
+        # A. Instance or class, to class
+        element = (
+            type(element)
+            if (
+                not PythonValidator.is_a_class(element) and
+                PythonValidator.is_an_instance(element)
+            ) else
+            element
         )
+            
+        # Force list to simplify checkings
+        cls = (
+            [cls]
+            if not PythonValidator.is_list(cls) else
+            cls
+        )
+            
+        # 1. Multiple values and they are all classes
+        if all(
+            PythonValidator.is_a_class(cls_item)
+            for cls_item in cls
+        ):
+            return any(
+                issubclass(element, cls_item)
+                for cls_item in cls
+            )
+
+        # 2. Multiple values and they are all strings
+        if all(
+            PythonValidator.is_string(cls_item)
+            for cls_item in cls
+        ):
+            # Check if nothing in common
+            return not {
+                getattr(base_class, '__name__', None)
+                for base_class in getattr(element, '__bases__', [])
+            }.isdisjoint(cls)
+
+        # 3. Multiple values and they are mixed classes and strings
+        if all(
+            (
+                PythonValidator.is_string(cls_item) or
+                PythonValidator.is_a_class(cls_item)
+            )
+            for cls_item in cls
+        ):
+            element_parents_class_names = getattr(element, '__bases__', None)
+
+            cls_str_list = [
+                cls_item
+                for cls_item in cls
+                if PythonValidator.is_string(cls_item)
+            ]
+            cls_classes_list = [
+                cls_item
+                for cls_item in cls
+                if PythonValidator.is_a_class(cls_item)
+            ]
+
+            return (
+                any(
+                    issubclass(element, cls_item)
+                    for cls_item in cls_classes_list
+                ) or
+                any(
+                    element_parent_class_name in cls_str_list
+                    for element_parent_class_name in element_parents_class_names
+                )
+            )
+        
+        # TODO: Maybe some parameter provided is invalid
+        return False
 
     @staticmethod
     def is_a_function(
@@ -390,7 +418,25 @@ class PythonValidator:
         return (
             PythonValidator.is_list(element) and
             all(
-                PythonValidator.is_instance(item, cls)
+                PythonValidator.is_instance_of(item, cls)
+                for item in element
+            )
+        )
+    
+    @staticmethod
+    def is_list_of_subclasses_of(
+        element: list[object],
+        cls: type
+    ) -> bool:
+        """
+        Check if the provided 'element' is a list in which
+        all the items are subclasses of the provided 'cls'
+        classes.
+        """
+        return (
+            PythonValidator.is_list(element) and
+            all(
+                PythonValidator.is_subclass_of(item, cls)
                 for item in element
             )
         )
@@ -411,6 +457,7 @@ class PythonValidator:
             )
         )
 
+    @staticmethod
     def is_list_of_numbers(
         element: list[Union[int, float]]
     ) -> bool:
@@ -426,6 +473,7 @@ class PythonValidator:
             )
         )
 
+    @staticmethod
     def is_list_of_positive_numbers(
         element: list[Union[int, float]],
         do_include_zero: bool = True
@@ -443,6 +491,40 @@ class PythonValidator:
                 for item in element
             )
         )
+
+    @staticmethod
+    def is_list_of_int(
+        element: list[int]
+    ) -> bool:
+        """
+        Check if the provided 'element' is a list in which
+        all the items are int numbers.
+        """
+        return (
+            PythonValidator.is_list(element) and
+            all(
+                # TODO: Repeated on NumberValidator
+                PythonValidator.is_instance_of(item, int)
+                for item in element
+            )
+        )
+    
+    @staticmethod
+    def is_list_of_float(
+        element: list[float]
+    ) -> bool:
+        """
+        Check if the provided 'element' is a list in which
+        all the items are float numbers.
+        """
+        return (
+            PythonValidator.is_list(element) and
+            all(
+                # TODO: Repeated on NumberValidator
+                PythonValidator.is_instance_of(item, float)
+                for item in element
+            )
+        )
     
     @staticmethod
     def is_list_of_classes(
@@ -485,7 +567,7 @@ class PythonValidator:
         """
         Check if the provided 'element' is a dict.
         """
-        return PythonValidator.is_instance(element, dict)
+        return PythonValidator.is_instance_of(element, dict)
 
     @staticmethod
     def is_tuple(
@@ -494,7 +576,7 @@ class PythonValidator:
         """
         Check if the provided 'element' is a tuple.
         """
-        return PythonValidator.is_instance(element, tuple)
+        return PythonValidator.is_instance_of(element, tuple)
     
     @staticmethod
     def is_tuple_or_list_or_array_of_n_elements(
@@ -525,7 +607,7 @@ class PythonValidator:
         """
         Check if the provided 'element' is a tuple, an array
         or a list containing 2 numeric elements between the
-        also given limits.
+        also given limits (the limits are included).
 
         This method is very useful to validate coordinates 
         or ranges.
@@ -540,8 +622,8 @@ class PythonValidator:
             return False
         
         return (
-            first_element_lower_limit < element[0] < first_element_upper_limit and
-            second_element_lower_limit < element[1] < second_element_upper_limit
+            first_element_lower_limit <= element[0] <= first_element_upper_limit and
+            second_element_lower_limit <= element[1] <= second_element_upper_limit
         )
     
     @staticmethod
@@ -551,7 +633,7 @@ class PythonValidator:
         """
         Check if the provided 'element' is a string (str).
         """
-        return PythonValidator.is_instance(element, str)
+        return PythonValidator.is_instance_of(element, str)
     
     @staticmethod
     def is_boolean(
@@ -560,7 +642,7 @@ class PythonValidator:
         """
         Check if the provided 'element' is a boolean (bool).
         """
-        return PythonValidator.is_instance(element, bool)
+        return PythonValidator.is_instance_of(element, bool)
     
     @staticmethod
     def is_number(
@@ -581,10 +663,25 @@ class PythonValidator:
         element: 'np.ndarray'
     ) -> bool:
         """
-        Check if the provided 'element' is an instance of the
-        numpy array np.ndarray.
+        Check if the provided 'element' is an
+        instance of the numpy array 'ndarray'.
         """
-        return PythonValidator.is_instance(element, 'np.ndarray')
+        return PythonValidator.is_instance_of(element, 'ndarray')
+
+    @staticmethod
+    def is_numpy_dtype(
+        element: 'np.dtype'
+    ) -> bool:
+        """
+        Check if the provided 'element' is a
+        numpy 'dtype' (like 'np.float32', 
+        'np.int16', etc.).
+        """
+        return (
+            PythonValidator.is_instance_of(element, type) and
+            getattr(element, '__module__', '').startswith('numpy') and
+            hasattr(element, 'dtype')
+        )
 
     @staticmethod
     def is_url(
@@ -610,8 +707,8 @@ class PythonValidator:
         """
         # TODO: I think it is 'EnumMeta' not Enum
         return (
-            PythonValidator.is_subclass(element, 'YTAEnum') or
-            PythonValidator.is_subclass(element, 'Enum')
+            PythonValidator.is_subclass_of(element, 'YTAEnum') or
+            PythonValidator.is_subclass_of(element, 'Enum')
         )
     
     @staticmethod
@@ -650,6 +747,27 @@ class PythonValidator:
         """
         return callable(element)
     
+    @staticmethod
+    def is_bytes(
+        element: bytes
+    ) -> bool:
+        """
+        Check if the provided 'element' is bytes or
+        not.
+        """
+        return PythonValidator.is_instance_of(element, bytes)
+
+    # TODO: I don't know if this method should be
+    # in 'yta_programming' library instead of here
+    def is_dependency_installed(
+        dependency: str
+    ) -> bool:
+        """
+        Check if the 'dependency' provided is installed
+        in this project or if not.
+        """
+        return importlib.util.find_spec(dependency) is not None
+    
     # TODO: This method below is not complete, need work
     @staticmethod
     def validate_method_params(
@@ -740,7 +858,7 @@ class PythonValidator:
             elif user_param == None:
                 raise Exception(f'The param "{str(param.name)}" has None value provided and we expected one of these types: {", ".join(types)}.')
             else:
-                if not PythonValidator.is_instance(user_param, types) and types is not NO_TYPE:
+                if not PythonValidator.is_instance_of(user_param, types) and types is not NO_TYPE:
                     print(types)
                     types_str = ', '.join(types)
                     raise Exception(f'The param value "{str(param.name)}" provided "{str(user_param)}" is not one of the expected types: {types_str}.')
