@@ -2,6 +2,7 @@
 from yta_validation import PythonValidator
 from yta_validation.number import NumberValidator
 from yta_validation.error_message import ErrorMessage
+from array import array
 from typing import Union
 
 
@@ -150,6 +151,32 @@ class ParameterValidator:
         ParameterValidator.validate_mandatory(name, value)
         ParameterValidator.validate_callable(name, value)
 
+    @staticmethod
+    def validate_bytes(
+        name: str,
+        value: bytes
+    ) -> None:
+        """
+        Validate if the provided 'value' is bytes.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_bytes(value)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_bytes(name))
+        
+    @staticmethod
+    def validate_mandatory_bytes(
+        name: str,
+        value: bytes
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None and
+        is bytes.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_bytes(name, value)
+
     @staticmethod
     def validate_int(
         name: str,
@@ -372,6 +399,95 @@ class ParameterValidator:
         ParameterValidator.validate_mandatory(name, value)
         ParameterValidator.validate_number_between(name, value, lower_limit, upper_limit)
 
+    @staticmethod
+    def validate_numpy_array(
+        name: str,
+        value: Union['np.ndarray', None]
+    ) -> None:
+        """
+        Validate if the provided 'value' is a numpy
+        array.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_numpy_array(value)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_numpy_array(name))
+        
+    @staticmethod
+    def validate_mandatory_numpy_array(
+        name: str,
+        value: 'np.ndarray'
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None
+        and is a numpy array.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_numpy_array(name, value)
+
+    @staticmethod
+    def validate_numpy_dtype(
+        name: str,
+        value: Union['np.dtype', None]
+    ) -> None:
+        """
+        Validate if the provided 'value' is a numpy
+        dtype.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_numpy_dtype(value)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_numpy_dtype(name))
+
+    @staticmethod
+    def validate_mandatory_numpy_dtype(
+        name: str,
+        value: 'np.dtype'
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None
+        and is a numpy dtype.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_numpy_dtype(name, value)
+
+    @staticmethod
+    def validate_tuple(
+        name: str,
+        value: Union[tuple, None],
+        n_elements: Union[int, None] = None
+    ) -> None:
+        """
+        Validate if the provided 'value' is a tuple of
+        'n_elements' (if provided).
+        """
+        if (
+            value is not None and
+            (
+                not PythonValidator.is_tuple(value) or
+                (
+                    n_elements is not None and
+                    len(value) != n_elements
+                )
+            )
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_tuple(name, n_elements))
+        
+    @staticmethod
+    def validate_mandatory_tuple(
+        name: str,
+        value: tuple,
+        n_elements: Union[int, None] = None
+    ):
+        """
+        Validate if the provided 'value' is not None
+        and is a tuple of 'n_elements' elements.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_tuple(name, value, n_elements)
+
     @staticmethod
     def validate_instance(
         name: str,
@@ -398,7 +514,7 @@ class ParameterValidator:
         """
         if (
             value is not None and
-            not PythonValidator.is_instance(value, cls)
+            not PythonValidator.is_instance_of(value, cls)
         ):
             raise Exception(ErrorMessage.parameter_is_not_instance_of(name, cls))
 
@@ -444,6 +560,65 @@ class ParameterValidator:
         ParameterValidator.validate_mandatory(name, value)
         ParameterValidator.validate_class_of(name, value, cls)
 
+    @staticmethod
+    def validate_subclass_of(
+        name: str,
+        value: type,
+        cls: list[Union[type, str]]
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None and is 
+        a subclass of one of the given 'cls' classes.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_subclass_of(value, cls)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_subclass_of(name, cls))
+
+    @staticmethod
+    def validate_mandatory_subclass_of(
+        name: str,
+        value: type,
+        cls: list[Union[type, str]]
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None and is 
+        a subclass of one of the given 'cls' classes.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_subclass_of(name, value, cls)
+
+    @staticmethod
+    def validate_list_of_subclasses_of(
+        name: str,
+        value: type,
+        cls: list[Union[type, str]]
+    ) -> None:
+        """
+        Validate if the provided 'value' is a subclass
+        of one of the given 'cls' classes.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_list_of_subclasses_of(value, cls)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_list_of_subclasses_of(name, cls))
+
+    @staticmethod
+    def validate_mandatory_list_of_subclasses_of(
+        name: str,
+        value: type,
+        cls: list[Union[type, str]]
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None
+        and is a subclass of one of the given 'cls'
+        classes.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_list_of_subclasses_of(name, value, cls)
+
     @staticmethod
     def validate_list_of_numbers(
         name: str,
@@ -499,6 +674,60 @@ class ParameterValidator:
         ParameterValidator.validate_mandatory(name, value)
         ParameterValidator.validate_list_of_positive_numbers(name, value, do_include_zero = do_include_zero)
 
+    @staticmethod
+    def validate_list_of_int(
+        name: str,
+        value: Union[list[int], None]
+    ) -> None:
+        """
+        Validate if the provided 'value' is a list of
+        int values.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_list_of_int(value)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_list_of_int(name))
+        
+    @staticmethod
+    def validate_mandatory_list_of_int(
+        name: str,
+        value: list[int]
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None and is a
+        list of int numbers.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_list_of_int(name, value)
+
+    @staticmethod
+    def validate_list_of_float(
+        name: str,
+        value: Union[list[float], None]
+    ) -> None:
+        """
+        Validate if the provided 'value' is a list of
+        float values.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_list_of_float(value)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_list_of_float(name))
+        
+    @staticmethod
+    def validate_mandatory_list_of_float(
+        name: str,
+        value: list[float]
+    ) -> None:
+        """
+        Validate if the provided 'value' is not None and is a
+        list of float numbers.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_list_of_float(name, value)
+
     @staticmethod
     def validate_list_of_string(
         name: str,
@@ -640,6 +869,43 @@ class ParameterValidator:
         ParameterValidator.validate_mandatory(name, value)
         ParameterValidator.validate_list_of_these_classes(name, value, cls)
 
+    @staticmethod
+    def validate_numeric_tuple_or_list_or_array_of_2_elements_between_values(
+        name: str,
+        value: Union[tuple, list, array],
+        first_element_lower_limit: float,
+        first_element_upper_limit: float,
+        second_element_lower_limit: float,
+        second_element_upper_limit: float
+    ):
+        """
+        Validate if the provided 'value' is a numeric
+        list, tuple or array of 2 elements that are
+        between the given limits.
+        """
+        if (
+            value is not None and
+            not PythonValidator.is_numeric_tuple_or_list_or_array_of_2_elements_between_values(value, first_element_lower_limit, first_element_upper_limit, second_element_lower_limit, second_element_upper_limit)
+        ):
+            raise Exception(ErrorMessage.parameter_is_not_numeric_tuple_or_list_or_array_of_2_elements_between_values(name, first_element_lower_limit, first_element_upper_limit, second_element_lower_limit, second_element_upper_limit))
+        
+    @staticmethod
+    def validate_mandatory_numeric_tuple_or_list_or_array_of_2_elements_between_values(
+        name: str,
+        value: Union[tuple, list, array],
+        first_element_lower_limit: float,
+        first_element_upper_limit: float,
+        second_element_lower_limit: float,
+        second_element_upper_limit: float
+    ):
+        """
+        Validate if the provided 'value' is not None and
+        a numeric list, tuple or array of 2 elements that
+        are between the given limits.
+        """
+        ParameterValidator.validate_mandatory(name, value)
+        ParameterValidator.validate_numeric_tuple_or_list_or_array_of_2_elements_between_values(name, value, first_element_lower_limit, first_element_upper_limit, second_element_lower_limit, second_element_upper_limit)
+
     @staticmethod
     def validate_dict_has_keys(
         name: str,
